% ========== HAPTIC BACKHOE CONTROLLER STARTUP ROUTINE =========
% Run this routine to setup the necessary workspace variables
% before building the Haptic Backhoe controller program in Simulink

clear all ; close all ; clc ;

Sampling_Time = 0.001;

% ------- Sensor Calibration ----------
% Enter trendline coefficients of the form:
%        y=ci(1)*x^2 + ci(2)*x + ci(3)

c1 = [-0.0009634 -0.4050766 2.3355377];     % swing sensor 2nd order trendline coeffs
c2 = [ -0.00644514 2.23812750 22.79925627];		% boom sensor 2nd order trendline coeffs
c3 = [-0.0057802 2.2376391 22.8266324];			% stick sensor 2nd order trendline coeffs
c4 = [-0.0020828 1.5286223 20.3434998];			% bucket sensor 2nd order trendline coeffs

save sensor_calib
clear all

% ------- Dimension Constants ----------

% Link lengths [in]
a1 = 8.5;
a2 = 48;
a3 = 38+11/16;
a4 = 18.5;			%????????

% Linear constants [in]
R1A = 10;
R1B = 35.875;
R1C = 17.5;
RAJ = 11/16;
R2B = 12+5/16;
R2C = 36;					% 36.125;
R2D = 9+7/8;				%10;
R2E = 8.625;
R3D = 47.25;
R3E = 34+5/16;
R3F = 4.5;
R3G = 5.5;
REF = 29.625;
RFH = 9.5;
RFI = 0.25;				%  ??????????????
R3I = sqrt(R3F^2 - RFI^2);
RGH = 9.5;
R4G = 19;     % ?????????????????

% Angular constants [rad]
T01A = acos(RAJ/R1A); 	%1.4682;   ???????????????
T12C = acos((a2^2 + R2C^2 - R1C^2)/(2*a2*R2C));  			% 0.3076;
T23D = 0.1317;
TB12 = acos((R1B^2 + a2^2 - R2B^2)/(2*a2*R1B));          %0.0842;
TD23 = acos((R2D^2 + a3^2 - R3D^2)/(2*a3*R2D));				% 2.5136;
TDFE = 0.0503;
TG34 = 1.6966;  %?????  acos((R3G^2 + a4^2 - R4G^2)/(2*R3G*a4));
T23E = acos((a3^2 + R3E^2 - R2E^2)/(2*a3*R3E));
alpha = asin((R3E*sin(acos((a3^2 + R3E^2 - R2E^2)/(2*a3*R3E))) - RFI)/REF);

save dimensions
load sensor_calib
load demo_trench

% ------- UDP Socket Setting ----------

% Setting with crossover setup
com.SendIP = '192.168.0.1';           
com.ReceiveIP = '192.168.0.1';                         % should  be '192.168.0.1', but set as "accept all"          

% Setting with Static Phantom IP and Static Feynman (Kontz's computer)
% com.SendIP = '128.61.140.190';                % Feynman computer
% com.ReceiveIP = '128.61.140.190';                  % Phantom computer


% General Settings
com.Ph2Bh_Port = 26401;                         % Port that is used for Phantom to Backhoe com.
com.Ph2Bh_Struct = {'double','uint32'};         % Data Structure for Phantom to Backhoe com.
com.Ph2Bh_Bytes = 40;                           % number of bytes in data structure
com.Ph2Bh_Port_Dim = {[4,1],[2,1]};             % number of variables passed

com.Bh2Ph_Port = 23201;                         % Port that is used for Backhoe to Phantom com.
com.Bh2Ph_Struct = {'double','uint32'};         % Data Structure for Backhoe to Phantom com.
                            
% Variables associated with A/D and D/A

% Input Filter coefficients
Butter_Order = 2;
Butter_Rad = 100;

% ------- Coordinate transfroms ----------
com.BhOrg_P  = [0 100 130]';        % location of mapped backhoe workspace in Phantom's workspace
com.sc = 25.4/12;                   % # mm/in divided by #inch in Backhoe per inch in Phantom

com.Bh2Ph_Transform = [ 0   -com.sc     0       com.BhOrg_P(1)
                        0       0   com.sc      com.BhOrg_P(2)
                    -com.sc     0       0       com.BhOrg_P(3)
                        0       0       0           1 ];

%com.Bh2Ph_Transform = eye(4);
                    
com.Ph2Bh_Transform = inv(com.Bh2Ph_Transform);